#include "main.h"

#define MAX_COLLECTABLES (5)
typedef struct
{
	int active;
	xy position;
	xy targetposition;
	xy initialposition;
	int animframe;
	int collected;
	int rescued;
} Collectable;
Collectable collectables[MAX_COLLECTABLES];

int totalcollected;
int totalspawned;
int survivalmultiplecountdown;

void initcollectables()
{
	int i;
	for(i=0; i<MAX_COLLECTABLES; i++)
	{
		collectables[i].active = 0;
		collectables[i].collected = 0;
		collectables[i].rescued = 0;
	}
	totalcollected = 0;
	totalspawned = 0;
	survivalmultiplecountdown = 0;
}

void scatterthecollected()
{
	int i;
	for(i=0; i<MAX_COLLECTABLES; i++)
	{
		if(!collectables[i].active || !collectables[i].collected)
			continue;
		collectables[i].targetposition = collectables[i].initialposition;
		collectables[i].collected = 0;
	}
	totalcollected = 0;
}

void reshuffleremoved(int collectedremoved)
{
	int i;
	for(i=0; i<MAX_COLLECTABLES; i++)
	{
		if(collectables[i].collected > collectedremoved)
			collectables[i].collected--;
	}
	totalcollected--;
}

void reshuffleadded()
{
	int i;
	for(i=0; i<MAX_COLLECTABLES; i++)
	{
		if(collectables[i].collected)
			collectables[i].collected++;
	}
	totalcollected++;
}

void updatecollectable(Collectable *collectable)
{
	if(!collectable->active)
		return;
	collectable->position.x = (collectable->position.x*4+collectable->targetposition.x)/5;
	collectable->position.y = (collectable->position.y*4+collectable->targetposition.y)/5;
	if(collectable->collected)
	{
		int historyindex = playerhistorypos - collectable->collected*SPRITE_SIZE;
		while(historyindex < 0)
			historyindex += PLAYER_HISTORY_LENGTH;
		collectable->targetposition = playerhistory[historyindex];
		
		// Check for rescue
		int worldx = (collectable->targetposition.x+SPRITE_SIZE/2)/SPRITE_SIZE;
		int worldy = (collectable->targetposition.y+SPRITE_SIZE/2)/SPRITE_SIZE;
		if(getspawner(worldx, worldy) == SPAWN_RESCUE)
		{
			if(!survival)
			{
				collectable->rescued = 1;
				collectable->targetposition.x = worldx*SPRITE_SIZE;
				collectable->targetposition.y = -1*SPRITE_SIZE;
				reshuffleremoved(collectable->collected);
				collectable->collected = 0;
			} else
			{
				if(survivalmultiplecountdown)
					survivaltime += 7500;
				else
					survivaltime += 5000;
				survivalmultiplecountdown = 32;
				collectable->targetposition = findblanktile();
				collectable->initialposition = collectable->targetposition;
				reshuffleremoved(collectable->collected);
				collectable->collected = 0;
			}
			playsound(SOUND_RESCUEMAN);
		}		
		// Check for hurt
		if(checkhurt(collectable->targetposition.x, collectable->targetposition.y))
		{
			collectable->targetposition.x = worldx*SPRITE_SIZE;
			collectable->targetposition.y = worldy*SPRITE_SIZE;
			reshuffleremoved(collectable->collected);
			collectable->collected = 0;
			playsound(SOUND_MANHURT);
		}
	} else
	{
		int xdist = collectable->targetposition.x - playerpos.x;
		int ydist = collectable->targetposition.y - playerpos.y;
		xdist *= xdist;
		ydist *= ydist;
		if(ydist < SPRITE_SIZE*SPRITE_SIZE && xdist < SPRITE_SIZE*SPRITE_SIZE)
		{
			reshuffleadded();
			collectable->collected = 1;
			playsound(SOUND_COLLECTMAN);	
		}
	}
	collectable->animframe = (collectable->animframe+1)%36;
}

void updatecollectables()
{
	int i;
	int rescued=0;
	for(i=0; i<MAX_COLLECTABLES; i++)
	{
		updatecollectable(&collectables[i]);
		if(collectables[i].active && collectables[i].rescued)
			rescued++;
	}
	if(rescued && rescued==totalspawned && !gameover)
	{
		gameover = 1;
		recordrecord(playtime);
		playsound(SOUND_WIN);
		nextleveltimer = 64;
	}
	if(survival && survivalmultiplecountdown)
		survivalmultiplecountdown--;
}

void drawcollectables()
{
	int i;
	for(i=0; i<MAX_COLLECTABLES; i++)
	{
		if(!collectables[i].active)
			continue;
		int yoffset;
		if(collectables[i].animframe/6<3)
			yoffset = collectables[i].animframe/6;
		else
			yoffset = 6-collectables[i].animframe/6;		
		drawsprite(SPRITE_MAN, collectables[i].position.x, collectables[i].position.y-yoffset);
	}
}

void spawnman(int x, int y)
{
	int i;
	for(i=0; i<MAX_COLLECTABLES; i++)
	{
		if(!collectables[i].active)
		{
			collectables[i].position.x = x;
			collectables[i].position.y = y;
			collectables[i].targetposition = collectables[i].position;
			collectables[i].initialposition = collectables[i].position;
			collectables[i].animframe = 0;
			collectables[i].active = 1;
			collectables[i].collected = 0;
			collectables[i].rescued = 0;
			totalspawned++;
			return;
		}
	}
}
