#include "main.h"

#include <GL/glu.h>
#include <GL/glext.h>

//gl reference for the font texture
GLuint fontex;

GLuint sprites;
GLuint block;

int XRES = 800;
int YRES = 600;
int PIXEL_SCALE=3;
int fullscreen=0;

/*
 * Draws a rectangle
 *
 * float x1, y1 topleft
 * float x2, y2 bottomright
 */
void drawbox(float x1,float y1,float x2,float y2) {
    glBegin(GL_QUADS);
        glVertex3f(x1*PIXEL_SCALE,y1*PIXEL_SCALE,1.0f);
        glVertex3f(x2*PIXEL_SCALE,y1*PIXEL_SCALE,1.0f);
        glVertex3f(x2*PIXEL_SCALE,y2*PIXEL_SCALE,1.0f);
        glVertex3f(x1*PIXEL_SCALE,y2*PIXEL_SCALE,1.0f);
    glEnd();
}
/*
 * Initialise the graphics
 */
void gfxinit() {
	int flags = SDL_OPENGL | SDL_HWSURFACE | SDL_DOUBLEBUF;
	SDL_WM_SetCaption("Only Forward", NULL);
	if(fullscreen)
		flags |= SDL_FULLSCREEN;
    SDL_SetVideoMode(XRES,YRES,0,flags);
    //setting area rendered to
    glViewport(0,0,XRES,YRES);
    //sets the blank colour (affects cls() )
    glClearColor(0.32f, 0.25f, 0.32f, 1.0f);
    //sets up a 'normal' (like most 2d library) projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0,XRES,YRES,0);
    
    //Load the font
    loadimage("gfx\\font.bmp", &fontex);
    //Load the sprites
    loadimage("gfx\\spritesheet.bmp", &sprites);
}
/*
 * Updates the screen (as we're using double buffering
 * this is 'flipping' to the screen).
 */
void gfxflip() {
    SDL_GL_SwapBuffers();
}
/*
 * Clears the screen with the ClearColor
 */
void cls() {
	if(survival)
		glClearColor(0.684f, 0.137f, 0.281f, 1.0f);
	else
		glClearColor(0.098f, 0.328f, 0.391f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
}
/*
 * Draws a single character to the screen
 *
 * charslot  the character (within the font, NOT ascii) to draw
 * float x, y
 */
void drawchar(int charslot,float x,float y) {
    //Enable textures (needed to draw a font, but needs to be off otherwise
    glEnable(GL_TEXTURE_2D);
    //Setup Additive blend
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    //whats this do david?
    glBindTexture(GL_TEXTURE_2D, fontex);
    //Calculate the row and coloumn within the font bitmap
    int row=(int)(charslot/8);
    int col=charslot-(row*8);
    //Draw the characters
    glBegin(GL_QUADS);
        glTexCoord2f(0.125*(float)col,0.125*(float)row);
        glVertex3f(x,y,1.0f);
        glTexCoord2f(0.125*(float)(col+1),0.125*(float)row);
        glVertex3f(x+FONT_SIZE,y,1.0f);
        glTexCoord2f(0.125*(float)(col+1),0.125*(float)(row+1));
        glVertex3f(x+FONT_SIZE,y+FONT_SIZE,1.0f);
        glTexCoord2f(0.125*(float)col,0.125*(float)(row+1));
        glVertex3f(x,y+FONT_SIZE,1.0f);
    glEnd();
    //Unset texturing and blending
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);
}


// For efficency, because we're using a single sprite sheet it's best to do multiple
// sprite draws within one glBegin->glEnd block.  This means we need to init/deinit
// manually before each sprite draw session.
void initspritedraw()
{
	//Enable textures 
    glEnable(GL_TEXTURE_2D);
    //Bind the neccessary texture
    glBindTexture(GL_TEXTURE_2D, sprites);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glBegin(GL_QUADS);
}

void drawscaledsprite(Sprite sprite, float x, float y, int scale)
{
	int row=(int)(sprite/SPRITE_SHEET_SIZE);
    int col=sprite-(row*SPRITE_SHEET_SIZE);
    int renderx = ((int)x)*PIXEL_SCALE;
    int rendery = ((int)y)*PIXEL_SCALE;
    int rendersize = SPRITE_RENDER_SIZE*scale;
    
    glTexCoord2f(((float)col)/SPRITE_SHEET_SIZE,((float)row)/SPRITE_SHEET_SIZE);
    glVertex3f(renderx,rendery,1.0f);
    glTexCoord2f(((float)col+1)/SPRITE_SHEET_SIZE,((float)row)/SPRITE_SHEET_SIZE);
    glVertex3f(renderx+rendersize,rendery,1.0f);
    glTexCoord2f(((float)col+1)/SPRITE_SHEET_SIZE,((float)row+1)/SPRITE_SHEET_SIZE);
    glVertex3f(renderx+rendersize,rendery+rendersize,1.0f);
    glTexCoord2f(((float)col)/SPRITE_SHEET_SIZE,((float)row+1)/SPRITE_SHEET_SIZE);
    glVertex3f(renderx,rendery+rendersize,1.0f);
}

void drawsprite(Sprite sprite, float x, float y)
{	
    drawscaledsprite(sprite, x, y, 1);
}

void deinitspritedraw()
{
	glEnd();
	//Unset texturing and blending
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);   
}

/*
 * Draws a string to the screen
 *
 * char  *string   a pointer to the string to be printed. MUST be properly terminated
 * float x, y
 */
void drawstring(char *string,float x,float y) {
	x *= PIXEL_SCALE;
	y *= PIXEL_SCALE;
    //Loop until out of string
    while (*string) {
        //upper-case letters (changed to lowercase within the font
        if (*string>=65&&*string<=90){	        
            drawchar(*string-65,x,y);
        //lower-case letters
        } else if (*string>=97&&*string<=122) {
            drawchar(*string-97,x,y);
        //numbers
        } else if (*string>=48&&*string<=57) {
            drawchar(*string-48+26,x,y);        
        // ! to /
        } else if (*string>=33&&*string<=47) {
	        drawchar(*string-33+36,x,y);
	    // : to @
        } else if (*string>=58&&*string<=64) {
	        drawchar(*string-58+51,x,y);	        
        }
        //shift x along and increment place in string
        x+=FONT_SIZE;
        string++;
    }
}

void drawshadowedstring(char *string, float x, float y, float r, float g, float b)
{
	setcol(0.2, 0.2, 0.2);
	drawstring(string, x+1, y+1);
	setcol(r, g, b);
	drawstring(string, x, y);
	setcol(1, 1, 1);
}


void loadimage(char *filename, GLuint *texture)
{
	SDL_Surface *tmp, *tmpb;
	tmp=SDL_LoadBMP(filename);
	tmpb=SDL_CreateRGBSurface(SDL_SWSURFACE,tmp->w,tmp->h,32,0xff,0xff00,0xff0000,0xff000000);	
	SDL_SetColorKey(tmp, SDL_SRCCOLORKEY, SDL_MapRGB(tmp->format, 255, 0, 255));
	
	SDL_SetAlpha(tmpb, 0,0);
	SDL_FillRect  (tmpb, NULL, 0);
	SDL_BlitSurface(tmp, NULL, tmpb, NULL);
   
    glGenTextures(1, texture);
    glBindTexture(GL_TEXTURE_2D, *texture);
    gluBuild2DMipmaps(GL_TEXTURE_2D, 4, tmpb->w, tmpb->h, GL_RGBA, GL_UNSIGNED_BYTE, tmpb->pixels);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);

    free (tmp->pixels);
    free (tmp);
}

/*
 * Set's the 'current' colour, will be used for polygons
 * until the colour is changed again.
 *
 * float r, g, b   red, green and blue colour components, 1.0f is max 0.0f is min
 */
void setcol(float r,float g,float b) {
	glDisable(GL_BLEND);
    glColor3f(r,g,b);
}

void setcol4(float r, float g, float b, float a)
{
	glEnable(GL_BLEND);
	glColor4f(r,g,b,a);
}
/*
 * Converts an angle and distance pair to an xy coordinate
 *
 * float angle, distance   the angle and distance of the point
 * returns xy              a struct containing an x and a y point
 */
xy angtoxy(float angle, float distance) {   //distance 0-1 angle degrees
    xy out;
    //Calculate x
    out.x=sin(RAD(angle))*distance;
    out.x=out.x*YRES*0.5+(XRES/2);
    //Calculate y
    out.y=cos(RAD(angle))*distance;
    out.y=out.y*YRES*0.5+(YRES/2);
    return out;
}

void formattime(char* buffer, int milliseconds)
{
	int minutes = milliseconds/60000;
	int seconds = (milliseconds-minutes*60000)/1000;
	int hundredthseconds = (milliseconds-minutes*60000-seconds*1000)/10;
	sprintf(buffer, "%.2d:%.2d:%.2d", minutes, seconds, hundredthseconds);
}

void setresolution(int x, int y, int pixelsize)
{
	XRES = x;
	YRES = y;
	PIXEL_SCALE = pixelsize;
    gfxinit();
}

void togglefullscreen()
{
	fullscreen = !fullscreen;
	gfxinit();
}
