#include "main.h"


//0 if the game isn't running
int running;
int frames;
int seconds;
int fps;
int showfps;
int gameover;
char openfilename[256];
int currentlevel;
int survival;
int nextleveltimer;

typedef struct
{
	char text[14];
	Sprite sprite;
} TutorialLine;
#define NUM_TUTORIALS (4)
#define MAX_TUTORIAL_LINES (24)
//        12345678901234
TutorialLine tutorials[NUM_TUTORIALS][24] =
{
	{
		{"Only Forward", SPRITE_MAX},
		{" ", SPRITE_MAX},
		{" ", SPRITE_MAX},
		{"Zip around the", SPRITE_MAX},
		{"roads with the", SPRITE_BLOBA},
		{"arrow keys.", SPRITE_MAX},
		{" ", SPRITE_MAX},
		{"Collect men by", SPRITE_MAN},
		{"touching them.", SPRITE_MAX},
		{" ", SPRITE_MAX},
		{"Bring all the", SPRITE_MAX},
		{"men back to", SPRITE_MAX},
		{"the safe zone", SPRITE_RESCUE},
		{"to complete", SPRITE_MAX},
		{"the level.", SPRITE_MAX},
		{"", SPRITE_MAX},
	},
	{
		{"Enemies will", SPRITE_MAX},
		{"make your life", SPRITE_MAX},
		{"harder.", SPRITE_MAX},
		{" ", SPRITE_MAX},
		{"This one hates", SPRITE_MAX},
		{"turning, and", SPRITE_MAX},
		{"will keep on", SPRITE_BLOBB},
		{"going straight", SPRITE_MAX},
		{"if it can.", SPRITE_MAX},
		{" ", SPRITE_MAX},
		{"You can toggle", SPRITE_MAX},
		{"sound with 's'", SPRITE_MAX},
		{"", SPRITE_MAX},
	},
	{
		{"This enemy", SPRITE_MAX},
		{"hates straight", SPRITE_MAX},
		{"lines, it will", SPRITE_BLOBD},
		{"turn whenever", SPRITE_MAX},
		{"possible.", SPRITE_MAX},
		{" ", SPRITE_MAX},
		{"The faster you", SPRITE_MAX},
		{"complete a", SPRITE_MAX},
		{"level the", SPRITE_MAX},
		{"better you are", SPRITE_MAX},
		{"doing!", SPRITE_MAX},
		{"", SPRITE_MAX},
	},
	{
		{"This final", SPRITE_MAX},
		{"enemy is slow,", SPRITE_BLOBC},
		{"but it is also", SPRITE_MAX},
		{"hard to shake.", SPRITE_MAX},
		{" ", SPRITE_MAX},
		{"After this you", SPRITE_MAX},
		{"are on your", SPRITE_MAX},
		{"own. Have fun!", SPRITE_MAX},
		{"", SPRITE_MAX},
	},
};

void drawtutorial()
{
	int i=0;
	while(strlen(tutorials[currentlevel][i].text) > 0)
	{
		drawshadowedstring(tutorials[currentlevel][i].text, (XRES/2)/PIXEL_SCALE+16, 8+8*i,1,1,1);
		if(tutorials[currentlevel][i].sprite != SPRITE_MAX)
		{
			initspritedraw();
			drawsprite(tutorials[currentlevel][i].sprite, (XRES/2)/PIXEL_SCALE, 4+8*i);
			deinitspritedraw();
		}
		i++;
	}
}

/*
 * Small routine to generate random numbers
 * between 0.0f and 1.0f
 */
float rnd() {
    return (float)(rand()%10000)/10000;
}

void updateopenfilename()
{
	sprintf(openfilename, "level%02d.lvl", currentlevel);
}

/*
 * Initialise graphics, input and sound
 */
void init() {
    //Randomize the random number generator    
    srand((unsigned int)time(NULL));
    //initialise SDL & the video subsystem
    SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_INIT_AUDIO);  
    //other init
    gfxinit();
    soundinit();
    inputinit();
    currentlevel = 0;
    updateopenfilename();
    readlevel(openfilename);
    loadrecords();
	running = 1;
	gameover = 1;
	seconds = 1;
	frames = 0;
	fps = -1;
	showfps = 0;
	nextleveltimer=-1;
}
/*
 * Perform a single 'logic frame'
 */
void dologic() {
	//Polls for events (currently only SDL_QUIT does anything)
    pollevents();
    updateworld();
    updatesound();
	if (keypressed[SDLK_ESCAPE])
    {
    	if(gameover)
   			running = 0;
   		else
			gameover = 1;
    }
    if (keypressed[SDLK_s])
    	togglesounds();
    if(!editing)
    {
	    if(!gameover)
	    {
    		updateplayer();
    		updateenemies();
	    } else
	    {
		    if(nextleveltimer != -1)
		    {
			    if(!nextleveltimer)
			    {
				    if(currentlevel < NUM_LEVELS-1)
		    			currentlevel++;
					updateopenfilename();
					readlevel(openfilename);
					gameover = 1;				    
			    }
			    nextleveltimer--;
		    } else
		    {
		    	if(keypressed[SDLK_SPACE])
		    	{
				    if(currentlevel < 5 || records[currentlevel-1] != INT_MAX)
				    {
		    			readlevel(openfilename);
		    			gameover = 0;
	    			}
	    		}
	    		if(keypressed[SDLK_DOWN] && currentlevel < NUM_LEVELS-1 && !survival)
	    		{
	    			currentlevel++;
		    		updateopenfilename();
    				readlevel(openfilename);
    				gameover = 1;
    	    	}
	    		if(keypressed[SDLK_UP] && currentlevel > 0 && !survival)
	    		{
		    		currentlevel--;
		    		updateopenfilename();
    				readlevel(openfilename);
    				gameover = 1;
    	    	}
    	    	if(keypressed[SDLK_RETURN])
    	    	{
	    		    if(records[4] != INT_MAX)
	    		    {
    	    			survival = !survival;
    	    			if(survival)
    	    				sprintf(openfilename, "survival.lvl");
    	    			else
    	    				updateopenfilename();
    	    			readlevel(openfilename);
    	    			gameover = 1;
	    			}
	    		}
    		}
		}
		updatecollectables();
	}
}
/*
 * Uninitialise sound and SDL
 */
void deinit() {
	sounddeinit();
    SDL_Quit();
}

int escapekeyrepeat = FALSE;
/*
 * Main method, where is all begins
 *
 * int argc       number of arguments  (not used)
 * char *argv[]   pointer to arguments (not used)
 */
int main(int argc, char *argv[])
{

    //Initialise the game
    init();   
    
    //Loop for game then menu then game then ....
    while (running)
    {
         //Gets the SDL ticks (number of milliseconds since SDL started)
        int phystime=SDL_GetTicks();

        //Game loop
        while (running)
        {
	        char buffer[256];
	        frames++;
	        if(SDL_GetTicks() > seconds*1000)
	        {
		        seconds++;
		        fps = frames;
		        frames = 0;
	        }
	        
            //'Catch up' with logic frames
            while (phystime<SDL_GetTicks())
            {
                dologic();
                phystime+=T_PHYS;
            }

            //Clear the screen ready for drawing
            cls();
            //Set the colour to draw everything in
            setcol(1.0f,1.0f,1.0f);
			// DO MAIN DRAWING HERE
			initspritedraw();
			drawworld();
			if(!editing)
			{
				drawcollectables();
				drawenemies();
				drawplayer();
			}
			deinitspritedraw();
			if(currentlevel < NUM_TUTORIALS && !editing && !survival)
				drawtutorial();	
			if(gameover)
			{
				int locked = !survival && currentlevel > 4 && records[currentlevel-1] == INT_MAX;
				char buffer[128];
				char bufferb[128];
				int ystart = (YRES/4)/PIXEL_SCALE*3-18;

				if(locked)
				{
					setcol4(0,0,0,0.8);
					drawbox(0,0, XRES/PIXEL_SCALE, YRES/PIXEL_SCALE);
				}
				setcol4(0,0,0,0.5);
				drawbox(0, ystart-2, XRES/PIXEL_SCALE, ystart+54);
				if(!survival)
				{					
					sprintf(buffer, "Level %02d", currentlevel+1);
					drawshadowedstring(buffer, (XRES/2)/PIXEL_SCALE-5*8, ystart,1,1,0.5);
					drawshadowedstring("- Press up/down to change -", (XRES/2)/PIXEL_SCALE-27*4, ystart+8,1,1,0.5);
					if(records[4] != INT_MAX)
						drawshadowedstring("- Press enter for survival mode -", (XRES/2)/PIXEL_SCALE-33*4, ystart+16,1,1,0.5);
					if(locked)
						drawshadowedstring("Locked!", (XRES/2)/PIXEL_SCALE-4*7, ystart+24,1,1,0.5);
					else
						drawshadowedstring("Press SPACE to play!", (XRES/2)/PIXEL_SCALE-20*4, ystart+28,1,1,1);
                	
					if(records[currentlevel] != INT_MAX)
					{
						formattime(buffer, records[currentlevel]);
						sprintf(bufferb, "Best Time: %s", buffer);
						if(playtime == records[currentlevel])
							drawshadowedstring(bufferb, (XRES/2)/PIXEL_SCALE-4*19, ystart+40, 1, 0.5, 0.5);
						else
							drawshadowedstring(bufferb, (XRES/2)/PIXEL_SCALE-4*19, ystart+40, 1, 1, 0.5);
					}
				} else
				{
					drawshadowedstring("Survival Mode", (XRES/2)/PIXEL_SCALE-13*4, ystart,1,1,0.5);
					drawshadowedstring("- Press enter for regular mode -", (XRES/2)/PIXEL_SCALE-33*4, ystart+8,1,1,0.5);
					drawshadowedstring("Press SPACE to play!", (XRES/2)/PIXEL_SCALE-20*4, ystart+28,1,1,1);
					formattime(buffer, records[NUM_LEVELS]);
					sprintf(bufferb, "Best Time: %s", buffer);
					if(playtime == records[currentlevel] && playtime)
						drawshadowedstring(bufferb, (XRES/2)/PIXEL_SCALE-4*19, ystart+40, 1, 0.5, 0.5);
					else
						drawshadowedstring(bufferb, (XRES/2)/PIXEL_SCALE-4*19, ystart+40, 1, 1, 0.5);					
				}
			}
			if(playtime > 0)
			{
				char buffer[128];
				if(!survival)
				{
					formattime(buffer, playtime);
					drawshadowedstring(buffer, (XRES/2)/PIXEL_SCALE-4*8, YRES/PIXEL_SCALE-8,1,1,1);
				} else
				{
					formattime(buffer, playtime);
					drawshadowedstring(buffer, 0, YRES/PIXEL_SCALE-8,1,1,1);
					formattime(buffer, survivaltime);
					drawshadowedstring(buffer, XRES/PIXEL_SCALE-8*8, YRES/PIXEL_SCALE-8,1,1,1);
				}
				
			}
			if(fps != -1)
			{	
				if(showfps)
				{
					sprintf(buffer, "%d fps", fps);
					drawshadowedstring(buffer,0,0,1,1,0.5);
				}
			}
            //Flip the screen
            gfxflip();
        }    
    }
    
    deinit();
    return(0);
}
