#ifndef _MAIN_H
#define _MAIN_H

#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <SDL/SDL.h>
#include <SDL/SDL_Image.h>
#include <GL/gl.h>
#include <limits.h>

//general
extern int running;
#define PI 3.1416
#define RAD(x) (x*PI/180.0f)
#define T_PHYS (1000/50)
#define TRUE (1)
#define FALSE (0)
typedef struct {
    float x;
    float y;
} xy;
typedef enum
{
	DIR_NONE=0,
	DIR_NORTH=1,
	DIR_EAST=2,
	DIR_SOUTH=4,
	DIR_WEST=8,
} Direction;
float rnd();
extern char openfilename[256];
extern int showfps;
extern int gameover;
extern int survival;
#define NUM_LEVELS (10)

//graphics
extern int XRES;
extern int YRES;
extern int PIXEL_SCALE;
extern int fullscreen;

#define SPRITE_SIZE 16 // 16x16
#define SPRITE_SHEET_SIZE 8 // 8x8
#define SPRITE_RENDER_SIZE (SPRITE_SIZE*PIXEL_SCALE)
#define FONT_SIZE (8*PIXEL_SCALE)

typedef enum
{
	SPRITE_ES, SPRITE_SW, SPRITE_EW, SPRITE_ESW, SPRITE_NSW, SPRITE_NESW, SPRITE_BLOBA, SPRITE_BLOBB,
	SPRITE_NE, SPRITE_NW, SPRITE_NS, SPRITE_NES, SPRITE_NEW, SPRITE_GRASS, SPRITE_BLOBC, SPRITE_BLOBD,
	SPRITE_MAN, SPRITE_RESCUE, SPRITE_MAST, SPRITE_SIGN, SPRITE_MAX,
} Sprite;

void cls();
void gfxinit();  //initialise gl graphics
void gfxflip();
void drawbox(float x1,float y1,float x2,float y2);
void loadimage(char *filename, GLuint *texture);
void drawstring(char *string,float x,float y);
void drawshadowedstring(char *string, float x, float y, float r, float g, float b);
void setcol(float r,float g,float b);
void setcol4(float r, float g, float b, float a);
void initspritedraw();
void drawscaledsprite(Sprite sprite, float x, float y, int scale);
void drawsprite(Sprite sprite, float x, float y);
void deinitspritedraw();
xy angtoxy(float angle,float distance);
void formattime(char* buffer, int milliseconds);
void setresolution(int x, int y, int pixelsize);
void togglefullscreen();

//input
extern Uint8 * key;
extern Uint8 keypressed[SDLK_LAST];
void pollevents();
void inputinit();

//sound
extern int musicplaying;
extern int playsounds;
typedef enum
{
	SOUND_BLIP0, SOUND_BLIP1, SOUND_BLIP2, SOUND_BLIP3, SOUND_COLLECTMAN, SOUND_RESCUEMAN,
	SOUND_PLAYERHURT, SOUND_MANHURT, SOUND_WIN, SOUND_MAX,
} Sound;
void soundinit();
void updatesound();
void sounddeinit();
void playmusic();
void stopmusic();
void playsound(Sound sound);
void togglesounds();
void togglemusic();
void mute(int on);

//world
#define WORLD_SIZE_X 16
#define WORLD_SIZE_Y 12
typedef struct
{
	Sprite sprite;
	int directions;
} Tile;
typedef enum
{
	TILE_BLANK, TILE_ES, TILE_SW, TILE_EW, TILE_ESW, TILE_NSW,
	TILE_NESW, TILE_NE,	TILE_NW, TILE_NS, TILE_NES, TILE_NEW,
	TILE_OFFICE, TILE_GRASS, TILE_MAST, TILE_MAX,
} TileType;
typedef struct
{
	Sprite sprite;
	void (*callback)(int x, int y);
} SpawnInfo;
typedef enum
{
	SPAWN_PLAYER, SPAWN_AI0, SPAWN_AI1, SPAWN_AI2, SPAWN_MAN, SPAWN_RESCUE, SPAWN_MAX,
} Spawner;
void mouseclick(int x, int y, int state, int button);
void updateworld();
void drawworld();
void readlevel(char *filename);
void writelevel(char *filename);
void blanklevel();
Direction getdirs(int x, int y);
Spawner getspawner(int x, int y);
xy findplayerpos();
xy findblanktile();
extern int nextleveltimer;
extern int editing;
extern int currentlevel;

//player
extern xy playerpos;
#define PLAYER_HISTORY_LENGTH (256)
extern xy playerhistory[PLAYER_HISTORY_LENGTH];
extern int playerhistorypos;
extern Direction directionchecks[4][4];
extern int playtime;
extern int survivaltime;
void initplayer();
void updateplayer();
void drawplayer();

//enemies
void initenemies();
void updateenemies();
void drawenemies();
void spawnenemy(int x, int y, int id);
int checkhurt(int x, int y);

//collectables
extern int totalcollected;
void initcollectables();
void updatecollectables();
void drawcollectables();
void scatterthecollected();
void spawnman(int x, int y);

//records
extern int records[NUM_LEVELS+1];
void loadrecords();
void saverecords();
void recordrecord(int time);
void recordsurvival(int time);

#endif
