#include "main.h"

int editing=0;
int inpalette=0;
int placingspawn=0;
TileType selectedtile=TILE_BLANK;

Tile tiles[TILE_MAX] =
{
	{SPRITE_MAX, DIR_NONE}, // TILE_BLANK
	{SPRITE_ES, DIR_EAST | DIR_SOUTH}, //TILE_ES
	{SPRITE_SW, DIR_SOUTH | DIR_WEST}, //TILE_SW
	{SPRITE_EW, DIR_EAST | DIR_WEST}, //TILE_EW
	{SPRITE_ESW, DIR_EAST | DIR_SOUTH | DIR_WEST}, //TILE_ESW
	{SPRITE_NSW, DIR_NORTH | DIR_SOUTH | DIR_WEST}, //TILE_NSW
	{SPRITE_NESW,  DIR_NORTH | DIR_EAST | DIR_SOUTH | DIR_WEST}, //TILE_NESW
	{SPRITE_NE, DIR_NORTH | DIR_EAST}, //TILE_NE
	{SPRITE_NW, DIR_NORTH | DIR_WEST}, //TILE_NW
	{SPRITE_NS, DIR_NORTH | DIR_SOUTH}, //TILE_NS
	{SPRITE_NES, DIR_NORTH | DIR_EAST | DIR_SOUTH}, //TILE_NES
	{SPRITE_NEW, DIR_NORTH | DIR_EAST | DIR_WEST}, //TILE_NEW	
	{SPRITE_GRASS,  DIR_NONE}, //TILE_OFFICE (now GRASS)
	{SPRITE_MAST,  DIR_NONE}, //TILE_MAST
	{SPRITE_SIGN,  DIR_NONE}, //TILE_SIGN
};
TileType world[WORLD_SIZE_Y][WORLD_SIZE_X];

void spwnplayer(int x, int y) {playerpos.x = x; playerpos.y = y;}
void spwnenemy0(int x, int y) {spawnenemy(x, y, 0);}
void spwnenemy1(int x, int y) {spawnenemy(x, y, 1);}
void spwnenemy2(int x, int y) {spawnenemy(x, y, 2);}
void spwnrescue(int x, int y) {} // do nothing
SpawnInfo spawninfo[SPAWN_MAX] =
{
	{SPRITE_BLOBA, spwnplayer},
	{SPRITE_BLOBB, spwnenemy0},
	{SPRITE_BLOBC, spwnenemy1},
	{SPRITE_BLOBD, spwnenemy2},
	{SPRITE_MAN, spawnman},
	{SPRITE_RESCUE, spwnrescue},
};
Spawner spawns[WORLD_SIZE_Y][WORLD_SIZE_X];

void mouseclick(int x, int y, int state, int button)
{
	if(state != SDL_PRESSED || !editing)
		return;
	int xindex = x/SPRITE_RENDER_SIZE;
	int yindex = y/SPRITE_RENDER_SIZE;
	if(inpalette)
	{
		placingspawn = 0;
		selectedtile = yindex*6+xindex;
		if(selectedtile >= TILE_MAX)
		{
			placingspawn = 1;
			selectedtile -= TILE_MAX;
			if(selectedtile > SPRITE_MAX)
				selectedtile = SPRITE_MAX;
		}
	} else
	{
		switch(button)
		{
			case SDL_BUTTON_LEFT:
				if(placingspawn)
					spawns[yindex][xindex] = selectedtile;
				else
					world[yindex][xindex] = selectedtile;
				break;
			case SDL_BUTTON_MIDDLE:
				selectedtile = world[yindex][xindex];
				break;
			case SDL_BUTTON_RIGHT:
				if(spawns[yindex][xindex] != SPAWN_MAX)
					spawns[yindex][xindex] = SPAWN_MAX;
				else
					world[yindex][xindex] = TILE_BLANK;
				break;
		}						
	}
}

void resetworld()
{
	int iy, ix;
	initplayer();
	initenemies();
	initcollectables();
	gameover = 0;
	for(iy=0; iy<WORLD_SIZE_Y; iy++)
		for(ix=0; ix<WORLD_SIZE_X; ix++)
			if(spawns[iy][ix] != SPAWN_MAX)
				spawninfo[spawns[iy][ix]].callback(ix*SPRITE_SIZE, iy*SPRITE_SIZE);
}

void updateworld()
{
	if(key[SDLK_LCTRL])
	{
		if(editing)
		{
			if(keypressed[SDLK_s])
				writelevel(openfilename);
			if(keypressed[SDLK_b])
				blanklevel();
		}
		if(keypressed[SDLK_e] && 0) /* Disabled editor for release */
		{
			editing = !editing;
			resetworld();
		}		
	}
	inpalette = editing && key[SDLK_SPACE];
}

void drawworld()
{
	int x,y;	
	if(inpalette)
	{
		int i;
		x=0;
		y=0;
		for(i=0; i<TILE_MAX; i++)
		{
			Sprite sprite = tiles[i].sprite;
			if(sprite != SPRITE_MAX)
				drawsprite(sprite, x*SPRITE_SIZE, y*SPRITE_SIZE);
			x++;
			if(x==6)
			{
				y++;
				x=0;
			}
		}
		for(i=0; i<SPAWN_MAX; i++)
		{
			Sprite sprite = spawninfo[i].sprite;
			if(sprite != SPRITE_MAX)
				drawsprite(sprite, x*SPRITE_SIZE, y*SPRITE_SIZE);
			x++;
			if(x==6)
			{
				y++;
				x=0;
			}
		}		
		return;
	}
	for(x=0; x<WORLD_SIZE_X; x++)
	{
		for(y=0; y<WORLD_SIZE_Y; y++)
		{
			Sprite sprite = tiles[world[y][x]].sprite;
			if(sprite != SPRITE_MAX)
				drawsprite(sprite, x*SPRITE_SIZE, y*SPRITE_SIZE);
			if(spawns[y][x] == SPAWN_RESCUE)
				drawsprite(spawninfo[SPAWN_RESCUE].sprite, x*SPRITE_SIZE, y*SPRITE_SIZE);
			else if(editing && spawns[y][x] != SPAWN_MAX)
			{
				sprite = spawninfo[spawns[y][x]].sprite;
				drawsprite(sprite, x*SPRITE_SIZE, y*SPRITE_SIZE);				
			}
		}
	}
}

Direction getdirs(int x, int y)
{
	return tiles[world[y/SPRITE_SIZE][x/SPRITE_SIZE]].directions;
}

Spawner getspawner(int x, int y)
{
	return spawns[y][x];
}

xy findplayerpos()
{
	xy out;
	int iy, ix;
	for(iy=0; iy<WORLD_SIZE_Y; iy++)
	{
		for(ix=0; ix<WORLD_SIZE_X; ix++)
		{
			if(spawns[iy][ix] == SPAWN_PLAYER)
			{
				out.x = ix*SPRITE_SIZE;
				out.y = iy*SPRITE_SIZE;
			}
		}
	}
	return out;
}

void blanklevel()
{
	int ix, iy;
	for(ix=0; ix<WORLD_SIZE_X; ix++)
	{
		for(iy=0; iy<WORLD_SIZE_Y; iy++)
		{
			world[iy][ix] = TILE_BLANK;
			spawns[iy][ix] = SPAWN_MAX;
		}
	}
	world[0][0] = TILE_ES;
	world[0][1] = TILE_SW;
	world[1][0] = TILE_NE;
	world[1][1] = TILE_NW;
	spawns[0][0] = SPAWN_PLAYER;
	resetworld();
}

void readlevel(char *filename)
{
	FILE *fp;
	int ix=0;
	int iy=0;
	
	char c;
	fp = fopen(filename, "r");
	if(!fp) // if file doesn't exist we're making a new level, so blank it out and return
	{
		blanklevel();
		return;
	}
	c = fgetc(fp);
	while(c != EOF)
	{		
		if(c-32 < TILE_MAX)
			world[iy][ix] = c-32;
		c = fgetc(fp);
		if(c-32 <= SPAWN_MAX)
			spawns[iy][ix] = c-32;
		ix++;
		if(ix==WORLD_SIZE_X)
		{
			ix = 0;
			iy++;	
		}
		c = fgetc(fp);
	}
	fclose(fp);
	sprintf(openfilename, "%s", filename);
	resetworld();
}

void writelevel(char *filename)
{
	FILE *fp;
	int ix;
	int iy;
	fp = fopen(filename, "w");
	for(iy = 0; iy<WORLD_SIZE_Y; iy++)
	{
		for(ix = 0; ix<WORLD_SIZE_X; ix++)
		{
			fputc(world[iy][ix]+32, fp); // The 32 offset is to avoid the ascii control chars (some of them do funny stuff)
			fputc(spawns[iy][ix]+32, fp); // The 32 offset is to avoid the ascii control chars (some of them do funny stuff)
		}
	}
	fclose(fp);
}

xy findblanktile()
{
	xy out;
	int run=1;
	while(run)
	{
		int x=rand()%WORLD_SIZE_X;
		int y=rand()%WORLD_SIZE_Y;
		if(tiles[world[y][x]].directions && spawns[y][x] == SPAWN_MAX)
		{
			out.x = x*SPRITE_SIZE;
		    out.y = y*SPRITE_SIZE;
		    run = 0;
	    }
    }
    return out;
}
